/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <math.h>
#include <string.h>
#include "mex.h"
#include "frame.h"
#include "stream.h"
#include "layer3.h"

#define INPUT_BUFFER_SIZE	(5*8192)
#define MP3_GRANULARITY 4600

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {

	/* Declarations */
	char *filename;
	int filename_len;
	FILE *mp3;
	struct mad_stream Stream;
	struct mad_frame Frame;
	unsigned char InputBuffer[INPUT_BUFFER_SIZE+MAD_BUFFER_GUARD];
	int i,gr;
	double save[576];
	double temp,temp2,sum;
	unsigned int frameNum;
	double *data = NULL;
	int data_size;
	unsigned int channel;

	/* Check number of arguments */
	if (nrhs<1) mexErrMsgTxt( "Not enough input arguments.");
	if (nlhs>1) mexErrMsgTxt( "Too many output arguments.");

	/* Input */
	filename_len = mxGetN(prhs[0])+1;
	filename = (char*)mxCalloc(filename_len, sizeof(char));
	mxGetString(prhs[0],filename,filename_len);
	mp3 = fopen(filename,"r");
	if (mp3==NULL) mexErrMsgTxt( "Cannot open file\n");
	channel = 0;
	if (nrhs>=2) channel = (unsigned int)mxGetPr(prhs[1])[0];
	if (channel>1) mexErrMsgTxt( "Channel parameter must be 0 or 1.");

	/* Decoding */
	mad_stream_init(&Stream);
	mad_frame_init(&Frame);
	frameNum = 0;
	if (nlhs>0) {
		data_size = 2*MP3_GRANULARITY;
		data = (double*) mxCalloc(data_size,sizeof(double));
	}
	for (i=0;i<576;i++) save[i] = 0.0;
	do {
		fail:
		if(Stream.buffer==NULL || Stream.error==MAD_ERROR_BUFLEN) {
			size_t ReadSize,Remaining;
			unsigned char *ReadStart;
			if(Stream.next_frame!=NULL) {
				Remaining=Stream.bufend-Stream.next_frame;
				memmove(InputBuffer,Stream.next_frame,Remaining);
				ReadStart=InputBuffer+Remaining;
				ReadSize=INPUT_BUFFER_SIZE-Remaining;
			} else { 
				ReadSize=INPUT_BUFFER_SIZE,
				ReadStart=InputBuffer,
				Remaining=0;
			}
			ReadSize=fread(ReadStart,1,ReadSize,mp3);
			if(ReadSize<=0) break;
			mad_stream_buffer(&Stream,InputBuffer,ReadSize+Remaining);
			Stream.error = MAD_ERROR_NONE;
		}
		Frame.options = Stream.options;
		if (!(Frame.header.flags & MAD_FLAG_INCOMPLETE) && mad_header_decode(&Frame.header, &Stream) == -1) goto fail;

		if (frameNum==0)
			if (Frame.header.layer!=3) mexErrMsgTxt( "File is not mp3.\n");

		Frame.header.flags &= ~MAD_FLAG_INCOMPLETE;
		mad_layer_III(&Stream, &Frame, 3);

		if (nlhs>0&&frameNum+2>=data_size) {
			data_size += 2*MP3_GRANULARITY;
			data = (double*) mxRealloc(data, data_size*sizeof(double));
		}

		for (gr=0;gr<2;gr++) {
			sum = 0.0;
			for (i=0;i<576;i++) {
				temp = Frame.xr[gr][channel][i]*Frame.xr[gr][channel][i];
				temp2 = temp - save[i];
				save[i] = temp;
				if (temp2<0) temp2 = - temp2;
				sum += sqrt(temp2);
			}
			if (nlhs>0) data[frameNum] = sum;
			frameNum++;
		}
	}while(1);

	/* End decoding */
	mad_frame_finish(&Frame);
	mad_stream_finish(&Stream);

	/* Output */
	if (nlhs>0) {
		plhs[0] = mxCreateDoubleMatrix(frameNum,1,mxREAL);
		memcpy(mxGetPr(plhs[0]),data,sizeof(double)*frameNum);
	}

	/* Clean */
	if (data) mxFree( data );
	fclose( mp3 );
	mxFree( filename );
}

